%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Funktion synth_add_huellkurve() 
%
%  Autor: Ulrich Stein, Dep. M+P, HAW Hamburg
%
%  Erzeugt:          22.04.2006
%  Letzte nderung:  23.03.2008
%
%--------------------------------------------------------------------
%
%  Addiert den Hllkurven-Effekt zu der Schwingung.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  schwingung = synth_add_huellkurve( orig )

    schwingung = orig;
    
    global synth_data;  
    
    if( synth_data.huellkurve.tattack  < eps &  ...
        synth_data.huellkurve.trelease < eps &  ...
        abs( synth_data.huellkurve.hattack -    ...
             synth_data.huellkurve.hrelease ) < eps ) 
         warndlg( 'Inkonsistente DAten in synth_add_huellkurve' );
         return;
    end
    
    effekt = orig;
    effekt(:) = 0;

    synth_data.huellkurve.tsustain = synth_data.dauer - ...
        synth_data.huellkurve.tattack - synth_data.huellkurve.trelease;
     
    % tsustain = synth_data.huellkurve.tsustain

    % Hllkurven-Parameter verwenden:
    %     tattack       Attack-Zeit in s
    %     hattack       Attack-Hhe, relativ
    %     tsustain      Sustain-Zeit in s
    %     trelease      Release-Zeit in s 
    %     hrelease      Release-Hhe, relativ
    TS = synth_data.dauer; 
    SR = synth_data.samplerate;
    endAttack  = ( SR * synth_data.huellkurve.tattack +1 );
    endSustain = ( SR * synth_data.huellkurve.tsustain + endAttack );
    endRelease = ( SR * synth_data.huellkurve.trelease + endSustain );
    H1 = synth_data.huellkurve.hattack;
    H2 = synth_data.huellkurve.hrelease;
    
    % Effekt-Feld anlegen, so lange wie orig:
    len = length(orig);  % = TS * SR + 1;
    if( len < endRelease )
      endRelease = len;
      if( len < endSustain )
         endSustain = len;
      end
    end
    
    % Attack-Phase
    attTime = endAttack;
    if( attTime > eps )
        for( s=1:endAttack )
           effekt(s) = s * H1 / attTime; 
        end
    end
    % Sustain-Phase
    susTime = endSustain-endAttack;
    if( susTime > eps )
        for( s=endAttack:endSustain )
           effekt(s) = H1 - (s-endAttack) * (H1-H2) / susTime; 
        end
    end
    % Release-Phase
    relTime = endRelease-endSustain;
    if( relTime > eps )
        for( s=endSustain:endRelease ) 
           effekt( int32(s) ) = H2 - (s-endSustain) * H2 / relTime; 
        end
    end
    
    % Original mit dem Effekt multiplizieren
    schwingung = orig .* effekt;

return;
